@@ -11,6 +11,7 @@ from pika.adapters.asyncio_connection import AsyncioConnection
 from s3i.exception import raise_error_from_response, raise_error_from_s3ib_amqp, S3IBrokerRESTError, S3IBrokerAMQPError
 from s3i.logger import APP_LOGGER
 from s3i.callback_manager import CallbackManager
+from s3i.config import Config
 
 CONTENT_TYPE = "application/json"
 HOST = "broker.s3i.vswf.dev"
@@ -63,29 +64,25 @@ class BrokerREST:
         self.headers["Authorization"] = "Bearer " + new_token
         self.headers_encrypted["Authorization"] = "Bearer " + new_token
 
-    def send(self, receiver_endpoints, msg, encrypted=False):
+    def send(self, endpoints, msg, encrypted=False):
         """
         Send a S³I-B message via S³I Broker API
-        :param receiver_endpoints: endpoints of the receivers
-        :type receiver_endpoints: list of str
+        :param endpoints: endpoints of the receivers
+        :type endpoints: list of str
         :param msg: message to be sent
         :type msg: str
         :param encrypted: if true, message will be sent encrypted, otherwise not.
         :type encrypted: bool
 
         """
-        end_points_encoded = []
-        for receiver_endpoint in receiver_endpoints:
-            encoded = urllib.parse.quote(receiver_endpoint, safe="")
-            end_points_encoded.append(encoded)
-
-        endpoints = ",".join(end_points_encoded)
-        url = "{}{}".format(self._url, endpoints)
-        headers = self.headers
         if encrypted:
             headers = self.headers_encrypted
-        response = requests.post(url=url, headers=headers, data=msg)
-        raise_error_from_response(response, S3IBrokerRESTError, 201)
+        else:
+            headers = self.headers
+        for endpoint in endpoints:
+            response = requests.post(url="{}/{}".format(self._url, endpoint), headers=headers, data=msg)
+            raise_error_from_response(response, S3IBrokerRESTError, 201)
+        return True
 
     def receive_once(self, queue):
         """
@@ -110,9 +107,10 @@ class BrokerAMQP:
     _ON_CHANNEL_OPEN = "_on_channel_open"
     _ON_CHANNEL_CLOSED = "_on_channel_closed"
 
-    def __init__(self, token, endpoint, callback, loop=asyncio.get_event_loop()):
+    def __init__(self, token, endpoint, callback, loop=asyncio.get_event_loop(), listenToEvents=False):
         self.__token = token
-        self.__endpoint = endpoint  # TODO Event Queue, more queue?
+        self.__endpoint = endpoint
+        self.__event_endpoint = None
         self.__loop = loop
         self.__callback = callback
         self.__credentials = None
@@ -120,6 +118,8 @@ class BrokerAMQP:
         self.__connection = None
         self.__channel = None
         self.__consumer_tag = None
+        self.__event_consumer_tag = None
+        self.__listen_to_events = listenToEvents
 
         self.__is_consuming = False
         self.__schedule_messages = []
@@ -162,6 +162,9 @@ class BrokerAMQP:
             on_close_callback=self.on_connection_closed,
             custom_ioloop=self.__loop
         )
+        if self.__listen_to_events:
+            self.create_event_queue()
+
     def on_connection_open(self, _unused_connection):
         APP_LOGGER.info("[S3I]: Connection to Broker built")
 
@@ -170,6 +173,7 @@ class BrokerAMQP:
         )
         self.__callbacks.process(
             self._ON_CONNECTION_OPEN,
+            self.__loop
         )
 
     @staticmethod
@@ -191,6 +195,7 @@ class BrokerAMQP:
             self.start_consuming()
         self.__callbacks.process(
             self._ON_CHANNEL_OPEN,
+            self.__loop
         )
 
     def add_on_channel_open_callback(self, callback, one_shot, *args, **kwargs):
@@ -198,6 +203,7 @@ class BrokerAMQP:
             self._ON_CHANNEL_OPEN,
             callback,
             one_shot,
+            False,
             *args,
             **kwargs
         )
@@ -207,6 +213,7 @@ class BrokerAMQP:
             self._ON_CONNECTION_OPEN,
             callback,
             one_shot,
+            False,
             *args,
             **kwargs
         )
@@ -218,9 +225,19 @@ class BrokerAMQP:
             queue=self.__endpoint,
             on_message_callback=self.__callback
         )
+        if self.__event_endpoint is not None:
+            self.__event_consumer_tag = self.__channel.basic_consume(
+                auto_ack=True,
+                exclusive=True,
+                queue=self.__event_endpoint,
+                on_message_callback=self.__callback)
+
         self.__is_consuming = True
 
     def stop_consuming(self):
+        if self.__event_consumer_tag is not None:
+            self.__channel.basic_cancel(self.__event_consumer_tag)
+
         cb = functools.partial(
             self.on_consumer_cancel_ok, userdata=self.__consumer_tag
         )
@@ -259,34 +276,89 @@ class BrokerAMQP:
             self.connect()
 
     def send(self, endpoints, msg):
-        try:
-            for endpoint in endpoints:
+        if isinstance(msg, dict):
+            msg = json.dumps(msg)
+        if self.__channel.is_open:
+            try:
+                for endpoint in endpoints:
+                    raise_error_from_s3ib_amqp(
+                        self.__channel.basic_publish,
+                        S3IBrokerAMQPError,
+                        DIRECT_EXCHANGE,
+                        endpoint,
+                        msg,
+                        pika.BasicProperties(
+                            content_type="application/json",
+                            delivery_mode=2
+                        ))
+                    APP_LOGGER.info("[S3I]: Sending message successes")
+
+            except S3IBrokerAMQPError as err:
+                APP_LOGGER.error("[S3I]: Sending message failed: {}".format(err))
+                return False
+            else:
+                return True
+
+    def publish_event(self, msg, topic):
+        if self.__channel.is_open:
+            try:
                 raise_error_from_s3ib_amqp(
                     self.__channel.basic_publish,
                     S3IBrokerAMQPError,
-                    DIRECT_EXCHANGE,
-                    endpoint,
+                    EVENT_EXCHANGE,
+                    topic,
                     msg,
                     pika.BasicProperties(
                         content_type="application/json",
-                        delivery_mode=2
                     ))
-                APP_LOGGER.info("[S3I]: Sending message successes")
+            except S3IBrokerAMQPError as err:
+                APP_LOGGER.error("[S3I]: Sending event failed: {}".format(err))
+                return False
+            else:
+                return True
 
-        except S3IBrokerAMQPError as err:
-            APP_LOGGER.error("[S3I]: Sending message failed: {}".format(err))
+    def create_event_queue(self):
+        conf = Config(self.__token)
+        identifier = self.__endpoint.replace("s3ibs://", '')
+        identifier = identifier.replace("s3ib://", '')
+        response = conf.create_broker_event_queue(thing_id=identifier, topic=[])
+        self.__event_endpoint = response.json()['queue_name']
+        return self.__event_endpoint
 
-    def publish_event(self, msg, topic):
-        try:
-            raise_error_from_s3ib_amqp(
-                self.__channel.basic_publish,
-                S3IBrokerAMQPError,
-                EVENT_EXCHANGE,
-                topic,
-                msg,
-                pika.BasicProperties(
-                    content_type="application/json",
-                ))
-        except S3IBrokerAMQPError as err:
-            APP_LOGGER.error("[S3I]: Sending event failed: {}".format(err))
+    def subscribe_topic(self, topic):
+        if self.__channel.is_open and self.__event_endpoint is not None:
+            try:
+                raise_error_from_s3ib_amqp(
+                    self.__channel.queue_bind,
+                    S3IBrokerAMQPError,
+                    exchange=EVENT_EXCHANGE,
+                    queue=self.__event_endpoint,
+                    routing_key=topic
+                )
+            except S3IBrokerAMQPError as err:
+                APP_LOGGER.error("[S3I]: Topic subscription failed: {}".format(err))
+                return False
+            else:
+                return True
+        else:
+            APP_LOGGER.error("[S3I]: No event endpoint configured yet")
+            return False
 
+    def unsubscribe_topic(self, topic):
+        if self.__channel.is_open and self.__event_endpoint is not None:
+            try:
+                raise_error_from_s3ib_amqp(
+                    self.__channel.queue_unbind,
+                    S3IBrokerAMQPError,
+                    exchange=EVENT_EXCHANGE,
+                    queue=self.__event_endpoint,
+                    routing_key=topic
+                )
+            except S3IBrokerAMQPError as err:
+                APP_LOGGER.error("[S3I]: Topic unsubscription failed: {}".format(err))
+                return False
+            else:
+                return True
+        else:
+            APP_LOGGER.error("[S3I]: No event endpoint configured yet")
+            return False
