@@ -4,10 +4,32 @@ import gnupg
 import pgpy
 from abc import ABC
 from s3i.exception import raise_error_from_s3ib_msg, S3IBMessageError
-
+from typing import Union, Optional, Dict, List
 GZIP_MAGIC_NUMBER = "1f8b"
 
 
+class TimeInterval:
+    def __init__(self, start: int, end: int):
+        self.__start = start
+        self.__end = end
+
+    @property
+    def start(self) -> int:
+        return self.__start
+
+    @start.setter
+    def start(self, value: int):
+        self.__start = value
+
+    @property
+    def end(self) -> int:
+        return self.__end
+
+    @end.setter
+    def end(self, value: int):
+        self.__end = value
+
+
 class Message(ABC):
     def __init__(self, base_msg=None, gzip_msg=None, pgp_msg=None):
         if isinstance(base_msg, dict):
@@ -132,7 +154,7 @@ class Message(ABC):
             raise TypeError("Only PGP Message is valid")
         cipher = pgpy.constants.SymmetricKeyAlgorithm.AES256
         session_key = cipher.gen_key()
-        self.pgp_msg = pub_key.encrypt(msg_pgp, cipher=cipher, sessionkey=session_key) # TODO: double print of logs
+        self.pgp_msg = pub_key.encrypt(msg_pgp, cipher=cipher, sessionkey=session_key)  # TODO: double print of logs
         return self.pgp_msg
 
     def decrypt(self, sec_key, msg_pgp):
@@ -252,7 +274,7 @@ class ServiceReply(Message):
                          results=None,
                          reply_to_endpoint=None,
                          replying_to_msg=None,
-                        ):
+                         ):
         _service_reply_json = {
             "sender": sender,
             "receivers": receivers,
@@ -288,13 +310,14 @@ class GetValueRequest(Message):
                             attribute_path,
                             reply_to_endpoint=None,
                             replying_to_msg=None,
+                            time_interval: Union[TimeInterval,Dict[str, int]] = None
                             ):
 
         _get_value_req = {
             "sender": sender,
             "receivers": receivers,
             "identifier": message_id,
-            "attributePath":attribute_path,
+            "attributePath": attribute_path,
             "messageType": "getValueRequest"
         }
 
@@ -304,6 +327,20 @@ class GetValueRequest(Message):
         if reply_to_endpoint is not None:
             _get_value_req["replyToEndpoint"] = reply_to_endpoint
 
+        if time_interval is not None:
+            if type(time_interval) == dict:
+                _get_value_req["timeInterval"] = {
+                    "start": int(time_interval["start"]),
+                    "end": int(time_interval["end"])
+                }
+            elif type(time_interval) == TimeInterval:
+                _get_value_req["timeInterval"] = {
+                    "start": time_interval.start,
+                    "end": time_interval.end
+                }
+            else:
+                raise TypeError()
+
         self.base_msg = _get_value_req
 
 
@@ -339,6 +376,88 @@ class GetValueReply(Message):
         self.base_msg = _get_value_reply
 
 
+class DeleteAttributeRequest(Message):
+    def __init__(self, base_msg=None, pgp_msg=None, gzip_msg=None):
+        super().__init__(
+            base_msg=base_msg,
+            pgp_msg=pgp_msg,
+            gzip_msg=gzip_msg
+        )
+
+    def fillDeleteAttributeRequest(self,
+                            sender,
+                            receivers,
+                            message_id,
+                            attribute_path,
+                            reply_to_endpoint=None,
+                            replying_to_msg=None,
+                            time_interval: Union[TimeInterval,Dict[str, int]] = None
+                            ):
+
+        _delete_attribute_req = {
+            "sender": sender,
+            "receivers": receivers,
+            "identifier": message_id,
+            "attributePath": attribute_path,
+            "messageType": "deleteAttributeRequest"
+        }
+
+        if replying_to_msg is not None:
+            _delete_attribute_req["replyingToMessage"] = replying_to_msg
+
+        if reply_to_endpoint is not None:
+            _delete_attribute_req["replyToEndpoint"] = reply_to_endpoint
+
+        if time_interval is not None:
+            if type(time_interval) == dict:
+                _delete_attribute_req["timeInterval"] = {
+                    "start": int(time_interval["start"]),
+                    "end": int(time_interval["end"])
+                }
+            elif type(time_interval) == TimeInterval:
+                _delete_attribute_req["timeInterval"] = {
+                    "start": time_interval.start,
+                    "end": time_interval.end
+                }
+            else:
+                raise TypeError()
+
+        self.base_msg = _delete_attribute_req
+
+
+class DeleteAttributeReply(Message):
+    def __init__(self, base_msg=None, pgp_msg=None, gzip_msg=None):
+        super().__init__(
+            base_msg=base_msg,
+            pgp_msg=pgp_msg,
+            gzip_msg=gzip_msg
+        )
+
+    def fillDeleteAttributeReply(self,
+                          sender: str,
+                          receivers: List[str],
+                          message_id: str,
+                          ok: bool,
+                          reply_to_endpoint: str = None,
+                          replying_to_msg: str = None,
+                          ):
+        _delete_attribute_reply = {
+            "sender": sender,
+            "receivers": receivers,
+            "identifier": message_id,
+            "ok": ok,
+            "messageType": "deleteAttributeReply"
+        }
+        if replying_to_msg is not None:
+            _delete_attribute_reply["replyingToMessage"] = replying_to_msg
+
+        if reply_to_endpoint is not None:
+            _delete_attribute_reply["replyToEndpoint"] = reply_to_endpoint
+
+        self.base_msg = _delete_attribute_reply
+
+
+
 class SetValueRequest(Message):
     def __init__(self, base_msg=None, pgp_msg=None, gzip_msg=None):
         super().__init__(
@@ -363,7 +482,7 @@ class SetValueRequest(Message):
             "identifier": message_id,
             "attributePath": attribute_path,
             "newValue": new_value,
-            "messageType":"setValueRequest"
+            "messageType": "setValueRequest"
         }
 
         if replying_to_msg is not None:
@@ -408,7 +527,7 @@ class SetValueReply(Message):
         self.base_msg = _set_value_reply
 
 
-class CustomEventRequest(Message):
+class SubscribeCustomEventRequest(Message):
     def __init__(self, base_msg=None, pgp_msg=None, gzip_msg=None):
         super().__init__(
             base_msg=base_msg,
@@ -416,34 +535,34 @@ class CustomEventRequest(Message):
             gzip_msg=gzip_msg
         )
 
-    def fillCustomEventRequest(self,
-                               sender,
-                               receivers,
-                               message_id,
-                               filter,
-                               attribute_paths,
-                               reply_to_endpoint=None,
-                               replying_to_msg=None,
-                               ):
+    def fillSubscribeCustomEventRequest(self,
+                                        sender,
+                                        receivers,
+                                        message_id,
+                                        filter,
+                                        attribute_paths,
+                                        reply_to_endpoint=None,
+                                        replying_to_msg=None,
+                                        ):
 
-        _custom_event_req = {
+        _subscribe_custom_event_req = {
             "sender": sender,
             "receivers": receivers,
             "identifier": message_id,
             "filter": filter,
             "attributePaths": attribute_paths,
-            "messageType": "customEventRequest"
+            "messageType": "subscribeCustomEventRequest"
         }
         if replying_to_msg is not None:
-            _custom_event_req["replyingToMessage"] = replying_to_msg
+            _subscribe_custom_event_req["replyingToMessage"] = replying_to_msg
 
         if reply_to_endpoint is not None:
-            _custom_event_req["replyToEndpoint"] = reply_to_endpoint
+            _subscribe_custom_event_req["replyToEndpoint"] = reply_to_endpoint
 
-        self.base_msg = _custom_event_req
+        self.base_msg = _subscribe_custom_event_req
 
 
-class CustomEventReply(Message):
+class SubscribeCustomEventReply(Message):
     def __init__(self, base_msg=None, pgp_msg=None, gzip_msg=None):
         super().__init__(
             base_msg=base_msg,
@@ -451,31 +570,99 @@ class CustomEventReply(Message):
             gzip_msg=gzip_msg
         )
 
-    def fillCustomEventReply(self,
-                             sender,
-                             receivers,
-                             message_id,
-                             topic,
-                             status,
-                             reply_to_endpoint=None,
-                             replying_to_msg=None,
-                             ):
-        _custom_event_reply = {
+    def fillSubscribeCustomEventReply(self,
+                                      sender,
+                                      receivers,
+                                      message_id,
+                                      topic,
+                                      status,
+                                      reply_to_endpoint=None,
+                                      replying_to_msg=None,
+                                      ):
+        _subscribe_custom_event_reply = {
             "sender": sender,
             "receivers": receivers,
             "identifier": message_id,
             "topic": topic,
             "status": status,
-            "messageType": "customEventReply"
+            "messageType": "subscribeCustomEventReply"
         }
 
         if replying_to_msg is not None:
-            _custom_event_reply["replyingToMessage"] = replying_to_msg
+            _subscribe_custom_event_reply["replyingToMessage"] = replying_to_msg
 
         if reply_to_endpoint is not None:
-            _custom_event_reply["replyToEndpoint"] = reply_to_endpoint
+            _subscribe_custom_event_reply["replyToEndpoint"] = reply_to_endpoint
 
-        self.base_msg = _custom_event_reply
+        self.base_msg = _subscribe_custom_event_reply
+
+
+class UnsubscribeCustomEventRequest(Message):
+    def __init__(self, base_msg=None, pgp_msg=None, gzip_msg=None):
+        super().__init__(
+            base_msg=base_msg,
+            pgp_msg=pgp_msg,
+            gzip_msg=gzip_msg
+        )
+
+    def fillUnsubscribeCustomEventRequest(self,
+                                          sender,
+                                          receivers,
+                                          message_id,
+                                          topic,
+                                          reply_to_endpoint=None,
+                                          replying_to_msg=None,
+                                          ):
+
+        _unsubscribe_custom_event_req = {
+            "sender": sender,
+            "receivers": receivers,
+            "identifier": message_id,
+            "topic": topic,
+            "messageType": "unsubscribeCustomEventRequest"
+        }
+        if replying_to_msg is not None:
+            _unsubscribe_custom_event_req["replyingToMessage"] = replying_to_msg
+
+        if reply_to_endpoint is not None:
+            _unsubscribe_custom_event_req["replyToEndpoint"] = reply_to_endpoint
+
+        self.base_msg = _unsubscribe_custom_event_req
+
+
+class UnsubscribeCustomEventReply(Message):
+    def __init__(self, base_msg=None, pgp_msg=None, gzip_msg=None):
+        super().__init__(
+            base_msg=base_msg,
+            pgp_msg=pgp_msg,
+            gzip_msg=gzip_msg
+        )
+
+    def fillUnsubscribeCustomEventReply(self,
+                                        sender,
+                                        receivers,
+                                        message_id,
+                                        topic,
+                                        status,
+                                        reply_to_endpoint=None,
+                                        replying_to_msg=None,
+                                        ):
+        _unsubscribeCustomEventReply = {
+            "sender": sender,
+            "receivers": receivers,
+            "identifier": message_id,
+            "topic": topic,
+            "status": status,
+            "messageType": "unsubscribeCustomEventReply"
+        }
+
+        if replying_to_msg is not None:
+            _unsubscribeCustomEventReply["replyingToMessage"] = replying_to_msg
+
+        if reply_to_endpoint is not None:
+            _unsubscribeCustomEventReply["replyToEndpoint"] = reply_to_endpoint
+
+        self.base_msg = _unsubscribeCustomEventReply
 
 
 class EventMessage(Message):
@@ -488,18 +675,16 @@ class EventMessage(Message):
 
     def fillEventMessage(self,
                          sender,
-                         receivers,
                          message_id,
-                         topic,
-                         timestamp,
-                         content,
+                         topic: str,
+                         timestamp: int,
+                         content: dict,
                          reply_to_endpoint=None,
                          replying_to_msg=None,
                          ):
 
         _event_message = {
             "sender": sender,
-            "receivers": receivers,
             "identifier": message_id,
             "topic": topic,
             "timestamp": timestamp,
@@ -513,4 +698,4 @@ class EventMessage(Message):
         if reply_to_endpoint is not None:
             _event_message["replyToEndpoint"] = reply_to_endpoint
 
-        self.base_msg = _event_message
\ No newline at end of file
+        self.base_msg = _event_message
