@@ -5,7 +5,7 @@ import uuid
 import hashlib
 import pyrql
 from jsonschema import validate
-from pyrql.query import And, Or, _Filter
+from pyrql.query import And, Or, Filter
 
 from s3i import EventMessage
 
@@ -33,12 +33,12 @@ class NamedEvent(object):
         _msg = EventMessage()
         _msg.fillEventMessage(
             sender=self.__topic.split(".")[0],
-            msg_id="s3i:{}".format(uuid.uuid4()),
+            message_id="s3i:{}".format(uuid.uuid4()),
             topic=self.topic,
             timestamp=int(time.time() * 1000.0),
             content=content
         )
-        self.__msg = _msg.msg
+        self.__msg = _msg.base_msg
 
 
 class CustomEvent(object):
@@ -52,6 +52,7 @@ class CustomEvent(object):
         self.__msg = {}
         self.__json_entry = json_entry
         self.__cur_attributes_dict = {}
+        self.__content_old = {}
 
     @property
     def filter_object(self):
@@ -95,17 +96,18 @@ class CustomEvent(object):
         for i in self.__attribute_paths:
             var = _uriToData(i, self.__json_entry)
             _content[i] = var
-
+        if _content == self.__content_old:
+            return
         _msg = EventMessage()
         _msg.fillEventMessage(
             sender=self.__topic.split(".")[0],
-            msg_id="s3i:{}".format(uuid.uuid4()),
+            message_id="s3i:{}".format(uuid.uuid4()),
             topic=self.__topic,
             timestamp=int(time.time() * 1000.0),
             content=_content
         )
-
-        self.__msg = _msg.msg
+        self.__msg = _msg.base_msg
+        self.__content_old = _content
 
 
 class EventManager:
@@ -118,6 +120,14 @@ class EventManager:
     def json_entry(self):
         return self.__json_entry
 
+    @property
+    def named_event_dict(self):
+        return self.__named_event_dict
+
+    @property
+    def custom_event_dict(self):
+        return self.__custom_event_dict
+
     @json_entry.setter
     def json_entry(self, value):
         if isinstance(value, dict):
@@ -166,21 +176,12 @@ class EventManager:
             return False
         _deleted_obj = self.__custom_event_dict.pop(topic)
         del _deleted_obj
+        return True
 
     def emit_named_event(self, publisher, topic, content):
-        """
-        """
+
         event_obj = self.__named_event_dict.get(topic)
 
-        """
-        if isinstance(event_obj, NamedEvent):
-            event_obj.generate_event_msg(content=content)
-            publisher.send(
-                msg=json.dumps(event_obj.msg),
-                receiver_endpoints=[topic],
-                exchange="eventExchange"
-            )
-        """
         if isinstance(event_obj, NamedEvent):
             event_obj.generate_event_msg(content=content)
             publisher.publish_event(
@@ -258,7 +259,7 @@ class RQL:
 
         """
         if self.__parsed_rql_expression.get("name") in self.__rql_math_operator:
-            rql_filter = _Filter(self.__parsed_rql_expression.get("name"),
+            rql_filter = Filter(self.__parsed_rql_expression.get("name"),
                                 self.__parsed_rql_expression.get("args")[0],
                                 self.__parsed_rql_expression.get("args")[1])
 
@@ -284,7 +285,7 @@ class RQL:
         for i in range(len(args)):
             if args[i].get("name") not in self.__rql_logic_operator:
                 filters.append(
-                    _Filter(args[i].get("name"), args[i].get("args")[0], args[i].get("args")[1]))
+                    Filter(args[i].get("name"), args[i].get("args")[0], args[i].get("args")[1]))
             else:
                 filters.append([])
                 self.__create_single_filter(args=args[i].get("args"), filters=filters[i])
