@@ -155,6 +155,7 @@ def raise_error_from_s3ib_msg(msg, error):
             "messageType": And(str, lambda s: s == "getValueRequest"),
             Optional("replyToEndpoint"): And(str, lambda n: "s3ib" in n),
             Optional("replyingToMessage"): And(str, lambda n: "s3i" in n),
+            Optional("timeInterval"): {"start": int, "end": int},
             "attributePath": str
         }, ignore_extra_keys=False
     )
@@ -223,6 +224,7 @@ def raise_error_from_s3ib_msg(msg, error):
             "messageType": And(str, lambda s: s == "deleteAttributeRequest"),
             Optional("replyToEndpoint"): And(str, lambda n: "s3ib" in n),
             Optional("replyingToMessage"): And(str, lambda n: "s3i" in n),
+            Optional("timeInterval"): {"start": int, "end": int},
             "attributePath": str,
         }, ignore_extra_keys=False
     )
@@ -238,25 +240,50 @@ def raise_error_from_s3ib_msg(msg, error):
         }, ignore_extra_keys=False
     )
 
-    custom_event_request_schema = Schema(
+    subscribe_custom_event_request_schema = Schema(
         {
             "sender": And(str, lambda n: "s3i" in n),
             "identifier": And(str, lambda n: "s3i:" in n),
             "receivers": And([And(str, lambda n: "s3i" in n)], lambda s: len(s) > 0),
-            "messageType": And(str, lambda s: s == "customEventRequest"),
+            "messageType": And(str, lambda s: s == "subscribeCustomEventRequest"),
             "filter": str,
-            "attributePath": And([str], lambda s: len(s) > 0),
+            "attributePaths": And([str], lambda s: len(s) > 0),
             Optional("replyToEndpoint"): And(str, lambda n: "s3ib" in n),
             Optional("replyingToMessage"): And(str, lambda n: "s3i" in n)
         }, ignore_extra_keys=False
     )
 
-    custom_event_reply_schema = Schema(
+    subscribe_custom_event_reply_schema = Schema(
         {
             "sender": And(str, lambda n: "s3i" in n),
             "identifier": And(str, lambda n: "s3i:" in n),
             "receivers": And([And(str, lambda n: "s3i" in n)], lambda s: len(s) > 0),
-            "messageType": And(str, lambda s: s == "customEventReply"),
+            "messageType": And(str, lambda s: s == "subscribeCustomEventReply"),
+            Optional("replyToEndpoint"): And(str, lambda n: "s3ib" in n),
+            Optional("replyingToMessage"): And(str, lambda n: "s3i" in n),
+            "status": Or(bool, str),
+            "topic": str
+        }, ignore_extra_keys=False
+    )
+
+    unsubscribe_custom_event_request_schema = Schema(
+        {
+            "sender": And(str, lambda n: "s3i" in n),
+            "identifier": And(str, lambda n: "s3i:" in n),
+            "receivers": And([And(str, lambda n: "s3i" in n)], lambda s: len(s) > 0),
+            "messageType": And(str, lambda s: s == "unsubscribeCustomEventRequest"),
+            "topic": str,
+            Optional("replyToEndpoint"): And(str, lambda n: "s3ib" in n),
+            Optional("replyingToMessage"): And(str, lambda n: "s3i" in n)
+        }, ignore_extra_keys=False
+    )
+
+    unsubscribe_custom_event_reply_schema = Schema(
+        {
+            "sender": And(str, lambda n: "s3i" in n),
+            "identifier": And(str, lambda n: "s3i:" in n),
+            "receivers": And([And(str, lambda n: "s3i" in n)], lambda s: len(s) > 0),
+            "messageType": And(str, lambda s: s == "unsubscribeCustomEventReply"),
             Optional("replyToEndpoint"): And(str, lambda n: "s3ib" in n),
             Optional("replyingToMessage"): And(str, lambda n: "s3i" in n),
             "status": Or(bool, str),
@@ -268,7 +295,6 @@ def raise_error_from_s3ib_msg(msg, error):
         {
             "sender": And(str, lambda n: "s3i" in n),
             "identifier": And(str, lambda n: "s3i:" in n),
-            "receivers": And([And(str, lambda n: "s3i" in n)], lambda s: len(s) > 0),
             "messageType": And(str, lambda s: s == "eventMessage"),
             "topic": str,
             "timestamp": int,
@@ -305,10 +331,14 @@ def raise_error_from_s3ib_msg(msg, error):
             return delete_attribute_request_schema.validate(msg)
         elif msg_type == "deleteAttributeReply":
             return delete_attribute_reply_schema.validate(msg)
-        elif msg_type == "customEventRequest":
-            return custom_event_request_schema.validate(msg)
-        elif msg_type == "customEventReply":
-            return custom_event_reply_schema.validate(msg)
+        elif msg_type == "subscribeCustomEventRequest":
+            return subscribe_custom_event_request_schema.validate(msg)
+        elif msg_type == "subscribeCustomEventReply":
+            return subscribe_custom_event_reply_schema.validate(msg)
+        elif msg_type == "unsubscribeCustomEventRequest":
+            return unsubscribe_custom_event_request_schema.validate(msg)
+        elif msg_type == "unsubscribeCustomEventReply":
+            return unsubscribe_custom_event_reply_schema.validate(msg)
         elif msg_type == "eventMessage":
             return event_message_schema.validate(msg)
         else:
@@ -332,5 +362,5 @@ def raise_error_from_response(response, error, expected_response_code):
         try:
             error_msg = response.json()
         except json.decoder.JSONDecodeError:
-            raise RuntimeError(response.text)
+            error_msg = response.text
     raise error(error_msg=error_msg)
